/***************************************************************************
 * dazzle.c
 *
 *            A simple graphics program demo for the Amiga by
 *            M. Peter Engelbrite
 *
 *	      A note about the name "dazzle"- back in the days of
 *	      MITS(Altair), SWTP and Processor Technology, there was
 *	      a color graphics board for the S-100 named the "Dazzler".
 *	      A computer program came with this board named "Dazzle".
 *	      That program had the same type of eight-fold symmetry that
 *	      my program does (although it was much simpler in the
 *	      generation of the patterns).  Legend has it that a 'mystery'
 *	      programmer wrote that program in a few hours, and sold it
 *	      to Chromemco (the manufacturer) for a song.  Whenever I
 *	      experiment with a graphics system,  I test it with some
 *	      version of this program and name it Dazzle in honor of that
 *	      mystery programmer.
 *
 *            Many thanks to Scott Ballantyne for "sparks", portions of
 *	      which were used in this program.
 *
 ************************************************************************/

/* Because of a wierd bug in the current Manx C compiler I am using
   (versions 1.99G), the comments below have been moved to their current
   location, from the end of the include file lines.  Fred Fish, 3/8/86 */

/* if you make any changes, please update the version number following the
   decimal */
/* This is now a SVGA program for MSC */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>

/* System dependent stuff */
#include <bios.h>
#include <dos.h>

typedef unsigned char byte;
typedef unsigned short word;
typedef unsigned long dword;

#define MAXX   (maxx) /* x dimension */
#define MAXY   (maxy) /* y dimension */
#define MAXINT 64 /* highest intensity of palette */
#define MAXPAL 256 /* size of palette */
#define MULT 2

int svga;

extern int _cdecl maxx, maxy;
extern int _cdecl whichvga(void);
extern void _cdecl svgamode(void);
extern void _cdecl mode13x(void);
extern void _cdecl point(word,word,word);
extern void _cdecl point13x(word,word,word);
extern word _cdecl rdpoint(word,word);
extern word _cdecl rdpoint13x(word,word);

#define MK_FP(s,o) ((void _far *) (((dword)(s) << 16) | (word)(o)))
#define pokeb(a,b,c) (*((byte _far*)MK_FP((a),(b))) = (byte)(c))
#define peekb(a,b)   (*((byte _far*)MK_FP((a),(b))))
#define PutPixel(x,y,c) (pokeb(0xa000,((y)*320)+(x),(c)))
#define GetPixel(x,y) (peekb(0xa000,((y)*320)+(x)))

#define WritePixel(x,y) point(x,y,color)
#define RangeRand(rng) (rand() % (rng))
#define clr_screen() svgamode()

#define bioskey(a) _bios_keybrd(a)

#pragma check_stack(off)

void GetPal(void _far *tp)
{
	_asm
	{
		les     di, tp;
		mov     dx,03C7h;
		xor     al,al;
		cli;
		out     dx,al;
		mov     cx,256 * 3;
		mov     dx,03C9h;
	getp_loop1:
		in      al,dx;
		and     al,3Fh;
		stosb;
		loop    getp_loop1;
	}
}

void SetPal(void _far *tp)
{
	_asm
	{
		cli;
		push    ds;
		lds     si,tp;

		mov     dx,03C8h;
		xor     al,al;
		out     dx,al;
		mov     cx,256 * 3;
		mov     dx,03C9h;
	adj_loop1:
		lodsb;
		out     dx,al;
		loop    adj_loop1;
		pop     ds;
		sti;
	}
}

#pragma check_stack()
/* End of system dependent stuff */

int PauseMode, TitleOn;
int offsa, offsb;
int color, cyclecnt;
unsigned char *Epal;

#define MAXS MAXY/2
#define HMAXS (MAXS/2)
#define QMAXS (MAXS/4)

#define POSY MAXY/2
#define POSX MAXX/2
#define INIT 1
#define EXEC 0

/* Find a new color similar to the given color */
#define COLORSTEP (7)
#define MINBRIGHT 1
#define MAXBRIGHT MAXINT
long findnewcolor(char *old)
{
	int red = old[0];
	int green = old[1];
	int blue = old[2];
	int r;

	r = rand();
	if (r & 8)
		red += r & COLORSTEP;
	else
		red -= r & COLORSTEP;
	r >>= 4;
	if (r & 8)
		green += r & COLORSTEP;
	else
		green -= r & COLORSTEP;
	r >>= 4;
	if (r & 8)
		blue += r & COLORSTEP;
	else
		blue -= r & COLORSTEP;

	if (blue > MAXBRIGHT)
		blue = MAXBRIGHT;
	else if (blue < MINBRIGHT)
		blue = MINBRIGHT;
	if (green > MAXBRIGHT)
		green = MAXBRIGHT;
	else if (green < MINBRIGHT)
		green = MINBRIGHT;
	if (red > MAXBRIGHT)
		red = MAXBRIGHT;
	else if (red < MINBRIGHT)
		red = MINBRIGHT;

	return (((long) blue) << 16) | (green << 8) | red;
}

/* these are utility functions for use in changing colors randomly select
   a new color, also randomizes the hue for that color */
void newcolor(void)
{
	long newcol;
	int color;

	color = RangeRand(MAXPAL-1)+1;
	newcol = findnewcolor(Epal+color * 3);
	Epal[color*3] = newcol & 0xff;
	Epal[color*3+1] = (newcol >> 8) & 0xff;
	Epal[color*3+2] = (newcol >> 16) & 0xff;
}

/* selects the next in sequence of available colors */
void nextcolor(void)
{
	if (++color > (MAXPAL-1))
		color = 1;
}

/* sets a specific color */
void setcolor(int coler)
{
	color = coler;
	if (color == 0)
		color = 1;
}

void generate_palette(void)
{

	Epal[0] = 0;
	Epal[1] = 0;
	Epal[2] = 0;
	Epal[3] = RangeRand(MAXINT-5)+5;
	Epal[4] = RangeRand(MAXINT-5)+5;
	Epal[5] = RangeRand(MAXINT-5)+5;

	for (color = 2; color < MAXPAL; color++) {
		long newcol;
		newcol = findnewcolor(Epal+((color-1)*3));
		Epal[color*3] = newcol & 0xff;
		Epal[color*3+1] = (newcol >> 8) & 0xff;
		Epal[color*3+2] = (newcol >> 16) & 0xff;
	}

	SetPal(Epal);
}

/* user definable graphics functions */

/* each function should set offsa and offsb and optionally chage the color
   it may maintain local variables, it should accept a value init which is
   set to TRUE to indicate that initialization is to occur.  offsa and
   offsb should range between 0 and 99 (overflow is not fatal, though).
   The function may call setcolor(n), newcolor(), or nextcolor() to change
   the color.  The name of your function must be added to the structure
   dotfunc in do_dazzle() at the end of this module. You will find that it
   is very easy to experiment and obtain interesting effects. */

void streamer(int init)
{
	static int cnt1, cnt2, cnt3;

	if (init) {
		cnt1 = cnt2 = 0;
		cnt3 = QMAXS;
	}
	if (cnt1++ > cnt2) {
		offsa -= cnt2;
		cnt1 = 0;
		cnt2 = cnt2 + 1;
		if (cnt2 > cnt3) {
			cnt2 = 0;
			if ((cnt3 = cnt3 + RangeRand(4)) > HMAXS)
				cnt3 = 0;
		}
		if (offsb++ >= MAXS)
			offsb = 0;
	}
	if (offsa++ >= MAXS)
		offsa = 0;
	if (RangeRand(1000) == 0)
		newcolor();
}

void lotsadots(int init)
{
	offsa = RangeRand(MAXS+1);
	offsb = RangeRand(MAXS+1);
	newcolor();
}

void ribbons(int init)
{
	static int cnt1, cnt2, cnt3;

	if (init) {
		cnt1 = cnt2 = 0;
		cnt3 = QMAXS;
	}
	if (cnt1++ > cnt2) {
		offsa -= cnt2;
		cnt1 = 0;
		cnt2 = cnt2 + 1;
		if (cnt2 > cnt3) {
			cnt2 = 0;
			if ((cnt3 = cnt3 + RangeRand(4)) > HMAXS)
				cnt3 = 0;
		}
		nextcolor();
		if (offsb++ >= MAXS)
			offsb = 0;
	}
	if (offsa++ >= MAXS)
		offsa = 0;
	if (RangeRand(1000) == 0)
		newcolor();
}


void scribble(int init)
{
	if ((offsa += RangeRand(3) - 1) > MAXS)
		offsa = 0;
	if ((offsb += RangeRand(3) - 1) > MAXS)
		offsb = 0;
	if (RangeRand(500) == 0)
		newcolor();
}

void curves(int init)
{
	static int wid, andx, bndx, aspeed, bspeed;

	if (init) {
		andx = bndx = wid = 0;
		aspeed = RangeRand(64) + 20;
		bspeed = RangeRand(64) + 20;
		/*
		aspeed = RangeRand(16*MAXS)+MAXS/5;
		bspeed = RangeRand(16*MAXS)+MAXS/5;
		*/
		newcolor();
	}
	if ((wid = (wid + 1) & 15) == 0) {
		offsa = (int)
			(sin((double) andx *.0005) * (double)(HMAXS-2)) + HMAXS-2;
		offsb = (int)
			(sin((double) bndx *.0005) * (double)(HMAXS-2)) + HMAXS-2;
		andx += aspeed;
		bndx += bspeed;
		if (andx > 32000)
			andx = 0;
		if (bndx > 32000)
			bndx = 0;
		nextcolor();
	} else
		offsa++;
}

void lines(int init)
{
	static int wid, aspeed, bspeed, ndx;

	if (init) {
		wid = ndx = 0;
		aspeed = bspeed = 1;
	}
	if ((wid = (wid + 1) & 3) == 0) {
		offsa += aspeed;
		ndx += bspeed;
		if (offsa < 0) {
			aspeed = 1;
			offsa = 0;
			if (RangeRand(3) == 0)
				newcolor();
		}
		if (ndx < 0) {
			bspeed = RangeRand(4) + 1;
			ndx = 0;
		}
		if (offsa >= (MAXS+1)) {
			aspeed = -1;
			offsa = MAXS;
		}
		if (ndx >= (MAXS-2)) {
			bspeed = 0 - (RangeRand(4) + 1);
			ndx = (MAXS-3);
		}
		offsb = ndx;
	} else
		offsb++;
}

void squares(int init)
{
	static int aend, astrt, sqcnt;

	if (init)
		sqcnt = 0;
	if (sqcnt != 0) {
		if (++offsa > aend) {
			offsa = astrt;
			offsb++;
			sqcnt--;
			if (offsb > MAXS) {
				sqcnt = 0;
				offsb = MAXS;
			}
			nextcolor();
		}
	}
	if (sqcnt == 0) {
		sqcnt = (RangeRand(10)) + 8;
		offsa = astrt = RangeRand(16*MAXS/25);
		aend = (RangeRand(16*MAXS/25)) + astrt + 8;
		if (aend > MAXS)
			aend = MAXS;
		while ((offsb = RangeRand(MAXS+QMAXS)) > MAXS);
		newcolor();
	}
}

void triangles(int init)
{
	static int aend, astrt, sqcnt;

	if (init)
		sqcnt = 0;
	if (sqcnt != 0) {
		if (++offsa > aend) {
			nextcolor();
			astrt++;
			aend--;
			if (astrt >= aend) {
				sqcnt = 0;
			} else {
				offsa = astrt;
				offsb++;
				if (offsb > MAXS) {
					sqcnt = 0;
					offsb = MAXS;
				}
			}
		}
	}
	if (sqcnt == 0) {
		sqcnt = 1;
		offsa = astrt = RangeRand(16*MAXS/25);
		aend = RangeRand(16*MAXS/25) + astrt + 8;
		if (aend > MAXS)
			aend = MAXS;
		offsb = RangeRand(MAXS+1);
		newcolor();
	}
}

void zigzags(int init)
{
	static int cnt, andx, bndx;

	if (init)
		cnt = -1;
	if (cnt-- <= 0) {
		andx = RangeRand(3) - 1;
		bndx = RangeRand(3) - 1;
		cnt = RangeRand(15);
		newcolor();
	}
	offsa += andx;
	offsb += bndx;
	if (offsa <= 0 || offsa >= MAXS)
		andx = -andx;
	if (offsb <= 0 || offsb >= MAXS)
		bndx = -bndx;
}

void grid(int init)
{
	static int aend, astrt, xcol, imagepnt, imagestrt;
	static char image[] =
	{

	 0, 0, 0, 1, 1, 0, 0, 0,
	 0, 0, 0, 1, 1, 0, 0, 0,
	 0, 0, 0, 0, 1, 1, 0, 0,
	 1, 1, 0, 0, 0, 1, 1, 1,
	 1, 1, 1, 0, 0, 0, 1, 1,
	 0, 0, 1, 1, 0, 0, 0, 0,
	 0, 0, 0, 1, 1, 0, 0, 0,
	 0, 0, 0, 1, 1, 0, 0, 0,

	 0, 0, 0, 1, 1, 0, 0, 0,
	 0, 0, 0, 1, 1, 0, 0, 0,
	 0, 0, 1, 1, 0, 0, 0, 0,
	 1, 1, 1, 0, 0, 0, 1, 1,
	 1, 1, 0, 0, 0, 1, 1, 1,
	 0, 0, 0, 0, 1, 1, 0, 0,
	 0, 0, 0, 1, 1, 0, 0, 0,
	 0, 0, 0, 1, 1, 0, 0, 0,

	 0, 0, 0, 1, 1, 0, 0, 0,
	 0, 0, 0, 1, 1, 0, 0, 0,
	 0, 0, 0, 1, 1, 0, 0, 0,
	 1, 1, 0, 1, 1, 0, 1, 1,
	 1, 1, 0, 1, 1, 0, 1, 1,
	 0, 0, 0, 1, 1, 0, 0, 0,
	 0, 0, 0, 1, 1, 0, 0, 0,
	 0, 0, 0, 1, 1, 0, 0, 0,

	 0, 0, 0, 1, 1, 0, 0, 0,
	 0, 0, 0, 1, 1, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0,
	 1, 1, 1, 1, 1, 1, 1, 1,
	 1, 1, 1, 1, 1, 1, 1, 1,
	 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 1, 1, 0, 0, 0,
	 0, 0, 0, 1, 1, 0, 0, 0,

	};

	if ((imagepnt & 63) == 0 || init) {
		imagestrt = RangeRand(4) * 64;
		offsa = RangeRand(MAXS-11);/*RangeRand(12) * 8;*/
		astrt = offsa;
		aend = offsa + 7;
		offsb = RangeRand(MAXS-11) /*RangeRand(12) * 8*/;
		imagepnt = imagestrt;
		if (RangeRand(25) == 0) {
			newcolor();
			xcol = RangeRand(QMAXS) + 5;
		}
		offsa--;
	}
	if (++offsa > aend) {
		offsa = astrt;
		offsb++;
	}
	color = (image[imagepnt++]) ? xcol : 0;
}

void diag1(int init)
{
	offsb--;
	if (offsa++ > (MAXS-1)) {
		offsa = 0;
		offsb++;
		nextcolor();
	}
	if (offsb < 0) {
		offsb = MAXS;
	}
}

void diag2(int init)
{
	offsb++;
	if (offsa++ > (MAXS-1)) {
		offsa = 0;
		offsb--;
		nextcolor();
	}
	if (offsb > MAXS) {
		offsb = 0;
	}
}

void diag3(int init)
{
	static int cnt, cnt2;

	if (init)
		cnt = cnt2 = 0;

	offsb++;
	if (offsa++ > (MAXS-1)) {
		offsa = 0;
		offsb--;
		color = 4;
		if (++cnt2 > 20)
			cnt2 = 0;
	}
	if (cnt++ > cnt2) {
		cnt = 0;
		if (color++ > 255)
			color = 4;
	}
	if (offsb > MAXS)
		offsb = 0;
}

void diag4(int init)
{
	offsb--;
	if (offsa++ > (MAXS-1)) {
		offsa = 0;
		offsb++;
	}
	if (offsb < 0) {
		offsb = MAXS;
	}
	setcolor(((offsa * offsb) >> 4) & 31);
}

void sweep(int init)
{
	static int ainc, alen;

	if (init || alen > /*51*/ (HMAXS+7)) {
		alen = ainc = offsa = offsb = 0;
	}
	if (ainc++ > alen) {
		offsb++;
		ainc = 0;
	}
	if (offsa++ >= MAXS) {
		nextcolor();
		alen++;
		offsa = offsb = 0;
	}
}

void fadein(int init)
{
	offsa = RangeRand(MAXS) + 1;
	offsb = RangeRand(MAXS) + 1;
	setcolor(offsa / offsb);
}

void spiral(int init)
{
	static int sidecnt, sidelen, colorcnt, dirndx, adir, bdir;
	static int newadir[] = {0, 1, 0, -1};
	static int newbdir[] = {1, 0, -1, 0};

	if (init || offsa >= MAXS || offsb >= MAXS) {
		sidecnt = MAXS/*100*/;
		sidelen = 0;
		dirndx = 0;
		offsa = RangeRand(MAXS);
		offsb = RangeRand(MAXS);
		colorcnt = 0;
	}
	if (sidecnt++ >= sidelen) {
		sidecnt = 0;
		dirndx = (dirndx + 1) & 3;
		if ((dirndx & 1) == 0)
			sidelen++;
		adir = newadir[dirndx];
		bdir = newbdir[dirndx];
	}
	offsa += adir;
	offsb += bdir;
	if ((colorcnt++ & 3) == 0)
		nextcolor();

}

void spiral1(int init)
{
	static int sidecnt, sidelen, dirndx, adir, bdir, size;
	static int newadir[] = {1, 1, -1, -1};
	static int newbdir[] = {1, -1, -1, 1};

	if (init || offsa >= MAXS || offsb >= MAXS || sidelen >= size) {
		sidecnt = /*100*/MAXS;
		sidelen = 0;
		dirndx = 0;
		offsa = RangeRand(MAXS);
		offsb = RangeRand(MAXS);
		size = RangeRand(/*70*/ 7*MAXS/10);
	}
	if (sidecnt++ >= sidelen) {
		sidecnt = 0;
		if (++dirndx > 3) {
			sidelen++;
			offsa--;
			dirndx = 0;
			nextcolor();
		}
		adir = newadir[dirndx];
		bdir = newbdir[dirndx];
	}
	offsa += adir;
	offsb += bdir;
}

void zigstream(int init)
{
	static int sidecnt, sidelen, colorcnt, dirndx, adir, bdir;
	static int newadir[] = {0, 1};
	static int newbdir[] = {1, 0};

	if (init || offsa >= MAXS || offsb >= MAXS) {
		sidecnt = /*100*/ MAXS;
		sidelen = 0;
		dirndx = 0;
		offsa = RangeRand(MAXS);
		offsb = RangeRand(MAXS);
		colorcnt = 0;
		setcolor(4);
	}
	if (sidecnt++ >= sidelen) {
		sidecnt = 0;
		if ((++dirndx & 1) == 0) {
			sidelen++;
			dirndx = 0;
		}
		adir = newadir[dirndx];
		bdir = newbdir[dirndx];
	}
	offsa += adir;
	offsb += bdir;
	if ((colorcnt++ & 3) == 0)
		nextcolor();

}

void formula1(int init)
{
	static int cona, conb, conx;

	if (init || offsa > MAXS) {
		offsa = 0;
		offsb = -1;
		cona = RangeRand(2) + 1;
		conb = RangeRand(3) + 1;
		conx = RangeRand(4) + 2;
	}
	if (offsb++ > offsa) {
		offsa++;
		offsb = 0;
	}
	setcolor(((offsb * offsb * conb) - (offsa << cona)) >> conx);
}

typedef struct ColorValue
{
	char Rvalue, Gvalue, Bvalue;
} ColorValue;

void rotatePalette(ColorValue *p, int n1, int n2, int d)
{
	register int i;

	for (i = n1; i <= n2; i++)
		p[i] = p[n1 + (i + d) % (n2 - n1 + 1)];
}

void cycolor(void)
{
	rotatePalette(Epal,1,255,1);
	SetPal(Epal);
	cyclecnt=100;
}

void do_dazzle(void)
{
	int (*currentfunc) (int);
	int type, typecnt, typetime;

	struct dotfuncs {
		int (*func) (int);
		int maxtime;
	};

	/* add the names of the custom graphics functions here */

	static struct dotfuncs dotfunc[] =
	{
	/* function name,    number of cycles to run (randomized) */

	 formula1, 15000,
	 lotsadots, 2000,
	 streamer, 6000,
	 ribbons, 6000,
	 curves, -6000,
	 lines, 4000,
	 squares, 6000,
	 triangles, 8000,
	 zigzags, 6000,
	 grid, 6000,
	 diag1, 8000,
	 diag2, 8000,
	 diag3, 12000,
	 diag4, 12000,
	 sweep, 3000,
	 fadein, 15000,
	 spiral, 15000,
	 spiral1, 15000,
	 zigstream, 10000
	};

	newcolor();
	cyclecnt = offsa = offsb = 0;
	typecnt = 100;
	typetime = 10;

	while (1) {
		if (bioskey(1))
			return;
		if (typecnt++ > typetime) {
			type = RangeRand((sizeof(dotfunc) / sizeof(struct dotfuncs)));
			typecnt = 0;

			if (dotfunc[type].maxtime < 0)
				typetime = RangeRand((-dotfunc[type].maxtime));
			else
				typetime = RangeRand (dotfunc[type].maxtime)*MULT;

			if (RangeRand(5) == 0) {
				/* clear screen here */
				clr_screen();
				generate_palette();
				newcolor();
			}
			currentfunc = dotfunc[type].func;
			(*currentfunc) (INIT);	/* initialize */
		}
		(*currentfunc) (EXEC);
		if (offsa > MAXS)
			offsa = MAXS;
		if (offsb > MAXS)
			offsb = MAXS;
		if (cyclecnt-- <= 0) {
			cycolor();
		}
		WritePixel(POSX + offsb, POSY + offsa);
		WritePixel(POSX + offsb, POSY - offsa);
		WritePixel(POSX - offsb, POSY + offsa);
		WritePixel(POSX - offsb, POSY - offsa);
		WritePixel(POSX + offsa, POSY + offsb);
		WritePixel(POSX + offsa, POSY - offsb);
		WritePixel(POSX - offsa, POSY + offsb);
		WritePixel(POSX - offsa, POSY - offsb);
	}
}

void setcolors(void)
{
	cprintf("Dazzle 1.1 by M. Peter Engelbrite\n\r");
	cprintf("Turbo C version by Gary F. Syck\n\r");
	cprintf("Quick C version by Jeff Carbonell\n\r");
	cprintf("This uses the VGAKIT library by John Bridges\n\r");
	cprintf("\n\rPress Any Key");
	bioskey(0);
	generate_palette();
	clr_screen();
}

int main(void)
{
	long t;

	svga=whichvga();

	if (!svga) {
		printf("Sorry but this program requires a known SVGA card\n\a");
		return(3);
	}

	Epal = malloc(3 * MAXPAL);
	clr_screen();
	setcolors();

	time(&t);
	srand(t);

	do_dazzle();
	bioskey(0);

	_asm mov ax,3;
	_asm int 10h;

	return(0);
}

